﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using CampusUOC.Clases;
using CampusUOC.Recursos;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;

namespace CampusUOC
{
    public partial class Asignatura : PhoneApplicationPage
    {
        private string _id;

        private Classroom _classroom;
        private Boards _boards;
        private Materials _materials;
        private ResourcesMapping _resources;
        private People _users;

        public ListBox LbPeople
        {
            get { return lbPeople; }
            set { lbPeople = value; }
        }

        public Boards Boards
        {
            set
            {
                _boards = value;
                lbBoards.ItemsSource = _boards.boards;
            }

            get
            {
                return _boards;
            }
        }

        public Materials Materials
        {
            set
            {
                _materials = value;
                lbMaterials.ItemsSource = _materials.materials;
            }

            get
            {
                return _materials;
            }
        }

        public ResourcesMapping Resources
        {
            set
            {
                _resources = value;
                lbResources.ItemsSource = _resources.resources;
            }

            get
            {
                return _resources;
            }
        }

        public People People
        {
            set
            {
                _users = value;
                lbPeople.ItemsSource = _users.users;
            }

            get
            {
                return _users;
            }
        }

        public Classroom Classroom
        {
            get
            {
                return this._classroom;
            }

            set
            {
                _classroom = value;
                if (value != null)
                {
                    lblSubject.Text = _classroom.shortTitle;
                    lblSubjectLong.Text = _classroom.title;
                    lblCode.Text = _classroom.code;
                    lbAssignaments.ItemsSource = _classroom.assignments;
                    int id;
                    if (int.TryParse(_classroom.id, out id))
                    {
                        CargaBoards(id);
                        CargaMaterials(id);
                        CargaPeople(id);
                        CargaResources(id);
                    }
                }
                else
                {
                    this.lblSubject.Text = "Null";
                    this.lblSubjectLong.Text = "Null";
                }

            }
        }

        public Asignatura()
        {
            InitializeComponent();

            CargarInterfaz();
        }

        private void CargarInterfaz()
        {
            lblApp.Text = AppResources.ApplicationTitle;
            lblSubject.Text = AppResources.Subject;
            lblTitleSubject.Text = AppResources.Subject;
            lblTitleCode.Text = AppResources.Code;
            panitemAssignaments.Header = AppResources.Assignaments;
            panitemBoards.Header = AppResources.Boards;
            //lblTitleAssignaments.Text = AppResources.Assignaments;
        }

        private void LayoutRoot_Loaded(object sender, RoutedEventArgs e)
        {

        }


        private async void CargaBoards(int idAsignatura)
        {
            Boards = await Procedimientos.GetBoards(idAsignatura);
        }

        private async void CargaPeople(int idAsignatura)
        {
            People = await Procedimientos.GetPeople(idAsignatura);
        }
        private async void CargaMaterials(int idAsignatura)
        {
            Materials = await Procedimientos.GetMaterials(idAsignatura);
        }
        private async void CargaResources(int idAsignatura)
        {
            Resources = await Procedimientos.GetResources(idAsignatura);
        }

        private async void CargaAsignatura()
        {
            Classroom = await Procedimientos.GetClassroom(_id);
            
        }

        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            if (this.NavigationContext.QueryString.ContainsKey("idSubject"))
            {
                _id = NavigationContext.QueryString["idSubject"];
                CargaAsignatura();
            }
        }

        private void lbAssignaments_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }

        private void lbMaterials_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(lbMaterials.SelectedIndex < 0) return;

            MessageBoxResult mbr = MessageBox.Show(AppResources.DoYouWantOpen + " " + _materials.materials[lbMaterials.SelectedIndex].url + "?", string.Empty, MessageBoxButton.OKCancel);
            if(mbr.Equals(MessageBoxResult.Cancel)) return;

            WebBrowserTask task = new WebBrowserTask();
            task.URL = _materials.materials[lbMaterials.SelectedIndex].url;
            task.Show();
             
        }

        private void lbBoards_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if(lbBoards.SelectedIndex < 0) return;

            CampusUOC.Clases.Board b = (CampusUOC.Clases.Board)lbBoards.SelectedItem;
            NavigationService.Navigate(new Uri("/Board.xaml?idSubject=" +_id + "&idBoard=" + b.id, UriKind.Relative));
            
        }

    }
}