﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;
using System.Windows.Controls;
using Newtonsoft.Json;
using RestSharp;
using System.Threading.Tasks;


namespace CampusUOC.Clases
{
    public static class Procedimientos
    {
        /*
        public static Licencia licencia
        {
            get { return LicenciaActual.Licencia; }
            set { LicenciaActual.Licencia = value; }
        }
        */

        public static void LlamadaAsincronica(RestClient cliente, RestRequest peticio, Action<string> callback)
        {
            EventWaitHandle Wait = new AutoResetEvent(false);

            //string resul = null;
            cliente.ExecuteAsync(
                peticio,
                response =>
                {
                    if (response.ResponseStatus == ResponseStatus.Completed)
                    {
                        Debug.WriteLine(response == null ? "Response null" : ("Contenido: " + response.Content ?? "Content null"));
                        callback(response.Content);
                        //resul = response.Content;
                    }
                    else
                    {
                        Debug.WriteLine(response.ErrorMessage);
                        callback(string.Empty);
                    }

                    //Wait.Set();
                });

            //Wait.WaitOne(1000*2);
        }

        public static Task<T> ExecuteTaskAsync<T>(this RestClient @this, RestRequest request)
            where T : new()
        {
            if (@this == null)
                throw new NullReferenceException();

            var tcs = new TaskCompletionSource<T>();

            @this.ExecuteAsync<T>(request, (response) =>
            {
                if (response.ErrorException != null)
                    tcs.TrySetException(response.ErrorException);
                else
                    tcs.TrySetResult(response.Data);
            });

            return tcs.Task;
        }

        private static Task<T> SelectAsync<T>(this RestClient client, IRestRequest request, Func<IRestResponse, T> selector)
        {
            var tcs = new TaskCompletionSource<T>();
            var loginResponse = client.ExecuteAsync(request, r =>
            {
                /*
                if (r.ErrorException == null)
                {
                    tcs.SetResult(selector(r));
                }
                else
                {
                    tcs.SetException(r.ErrorException);
                }
                 * */
                if (r.ResponseStatus == ResponseStatus.Completed)
                    tcs.TrySetResult(selector(r));
                else
                    tcs.TrySetException(r.ErrorException);

                Debug.WriteLine(r.Content??"content null");
            });
            return tcs.Task;
        }

        public static Task<string> GetContentAsync(this RestClient client, IRestRequest request)
        {
            return client.SelectAsync(request, r => r.Content);
        }

        public static Task<IRestResponse> GetResponseAsync(this RestClient client, IRestRequest request)
        {
            return client.SelectAsync(request, r => r);
        }

        public static async Task<T> ExecuteAsync<T>(RestRequest request)
        {
            RestClient _client = ConstruyeCliente();
            
            _client.Timeout = request.Timeout > 0 ? request.Timeout : 5*1000;
            
            var tcs = new TaskCompletionSource<T>();

            Debug.WriteLine(request.Resource);

            _client.ExecuteAsync(request, resp =>
            {
                if (resp.ResponseStatus != ResponseStatus.Completed)
                {
                    Debug.WriteLine(resp.ErrorMessage);
                    var ex = new Exception(resp.ErrorMessage);
                    tcs.TrySetException(ex);
                } 
                else
                {
#if DEBUG
                    Debug.WriteLine(typeof(T) + " => " + resp.Content);
                    //Debug.WriteLine(typeof(T) + " => " + StringEncodingConvert(resp.Content,"ISO-8859-1","UTF-8"));
#endif

                    if (resp.Content.StartsWith("<html><head><title>JBossWeb/2.0.1.GA - Error report</title><style>"))
                        tcs.TrySetException(new Exception(resp.Content));
                    else
                    {
                        var value = JsonConvert.DeserializeObject<T>(resp.Content);
                        tcs.TrySetResult(value);
                    }
                }

                
            });

            return await tcs.Task;
        }

        public static RestClient ConstruyeCliente()
        {
            return new RestClient(Config.BASEURI);
        }

        public static String StringEncodingConvert(String strText, String strSrcEncoding, String strDestEncoding)
        {
            System.Text.Encoding srcEnc = System.Text.Encoding.GetEncoding(strSrcEncoding);
            System.Text.Encoding destEnc = System.Text.Encoding.GetEncoding(strDestEncoding);
            byte[] bData = srcEnc.GetBytes(strText);
            byte[] bResult = System.Text.Encoding.Convert(srcEnc, destEnc, bData);
            return destEnc.GetString(bResult, 0, strText.Length);
        }

        public static RestRequest ConstruyePeticion(string uriFuncion, Dictionary<string, string> parametros, int timeOut)
        {
            return ConstruyePeticion(uriFuncion, parametros, timeOut, Method.GET, null);
        }

        public static RestRequest ConstruyePeticion(string uriFuncion, Dictionary<string, string> parametros, int timeOut, Method metodo, MessageMapping msg)
        {
            var peticio = new RestRequest(uriFuncion);
            if(timeOut > 0)
                peticio.Timeout = timeOut;
            peticio.AddParameter("access_token", LicenciaActual.Licencia.access_token);
            peticio.AddParameter("client_id", Config.ConsumerKey);
            peticio.AddParameter("client_secret", Config.ConsumerSecret);
            peticio.RequestFormat = DataFormat.Json;
            peticio.Method = metodo;
            
            foreach (var parametro in parametros)
            {
                peticio.AddParameter(parametro.Key, parametro.Value);
            }

            if (msg != null)
            {
                peticio.AddBody(msg);
            }

            return peticio;
        }

        public static async Task<bool> Post(String URI, String token, String JSONObject)
        {
            HttpClientHandler handler = new HttpClientHandler();
            var httpClient = new HttpClient(handler);
            
            HttpRequestMessage request = new HttpRequestMessage(HttpMethod.Post, Config.BASEURI + URI + "?access_token=" + token);
            var content = new StringContent(JSONObject);
            content.Headers.Remove("Content-Type");
            content.Headers.Add("Content-Type", "application/json");
            request.Content =  content;

            if (handler.SupportsTransferEncodingChunked())
            {
                request.Headers.TransferEncodingChunked = true;
            }

            //request.Headers.Add("content-type", "application/json");

            HttpResponseMessage response = await httpClient.SendAsync(request);
            return response.IsSuccessStatusCode;
        }

        public static RestRequest ConstruyePeticion(string uriFuncion, Dictionary<string, string> parametros)
        {
            return ConstruyePeticion(uriFuncion, parametros, 0);
        }


        public static async Task<InboxMapping> GetInbox()
        {
            return (await ExecuteAsync<InboxMapping>(ConstruyePeticion(Config.INBOX_URI, new Dictionary<string, string>())));
        }

        public static async Task<Eventos> GetEventos()
        {
            return (await ExecuteAsync<Eventos>(ConstruyePeticion(Config.CALENDAR_URI, new Dictionary<string, string>())));
        }

        public static async Task<Classrooms> GetClasses()
        {
            return (await ExecuteAsync<Classrooms>(ConstruyePeticion(Config.SUBJECTS_URI, new Dictionary<string, string>())));
        }

        public static async Task<Boards> GetBoards(int id)
        {
            return (await ExecuteAsync<Boards>(ConstruyePeticion(Config.SUBJECTS_URI + '/' + id + Config.BOARDS_SUB_URI, new Dictionary<string, string>())));
        }

        public static async Task<People> GetPeople(int id)
        {
            return (await ExecuteAsync<People>(ConstruyePeticion(Config.SUBJECTS_URI + '/' + id + Config.PEOPLE_SUB_URI, new Dictionary<string, string>())));
        }

        public static async Task<ResourcesMapping> GetResources(int id)
        {
            return (await ExecuteAsync<ResourcesMapping>(ConstruyePeticion(Config.SUBJECTS_URI + '/' + id + Config.RESOURCES_SUB_URI, new Dictionary<string, string>())));
        }

        public static async Task<Materials> GetMaterials(int id)
        {
            return (await ExecuteAsync<Materials>(ConstruyePeticion(Config.SUBJECTS_URI + '/' + id + Config.MATERIALS_SUB_URI, new Dictionary<string, string>())));
        }

        public static async Task<User> GetUser()
        {
            return (await ExecuteAsync<User>(ConstruyePeticion(Config.USER_URI, new Dictionary<string, string>())));
        }

        public static async Task<MessageMapping> GetBoardMessage(string idSubject, string idBoard, string idMessage)
        {
            Messages msgs = await GetBoardMessages(idSubject, idBoard, 0, 10000);
            foreach (var msg in msgs.messages)
            {
                if (msg.id == idMessage) return msg;
            }

            return null;
        }

        public static async Task<MessageMapping> SendMessage(string to, string subject, string body)
        {
            var parameters = new Dictionary<string, string>();
            /*
             * id 	string 	Message's identifier.
                subject 	string 	Message's subject.
                snippet 	string 	First characters of the message.
                date 	string 	Message's date.
                color 	long 	Color mark of the message.
                status 	long 	Message's status.
                from 	string 	Message's sender.
                to 	string 	Message's recipients.
                cc 	string 	Message's copy recipients.
                body 	string 	Message's body.
             * */

            MessageMapping msg = new MessageMapping();
            msg.body = body;
            msg.subject = subject;
            msg.to = to;
            msg.cc = string.Empty;
            msg.id = string.Empty;
            msg.snippet = body.Substring(0, Math.Min(body.Length, 100));
            //msg.date = "Sat Dec 28 18:28:40 CET 2013";
            return (await ExecuteAsync<MessageMapping>(ConstruyePeticion(Config.MAILMESSAGES_URI, parameters, 0, Method.PUT, msg)));
        }

        public static async Task<MessageMapping> GetMessage(string idMessage)
        {
            var parameters = new Dictionary<string, string>();
            parameters.Add("id", idMessage);
            return (await ExecuteAsync<MessageMapping>(ConstruyePeticion(Config.MAILMESSAGES_URI + '/' + idMessage, parameters)));
        }

        public static async Task<Classroom> GetClassroom(string idSubject)
        {
            var parameters = new Dictionary<string, string>();
            parameters.Add("id", idSubject);
            return (await ExecuteAsync<Classroom>(ConstruyePeticion(Config.SUBJECTS_URI + '/' + idSubject, parameters)));
        }

        public static async Task<Messages> GetBoardMessages(string domainId, string id, int start, int end)
        {
            var parameters = new Dictionary<string, string>();
            parameters.Add("id", id);
            parameters.Add("domain_id", domainId);
            parameters.Add("start", start.ToString());
            parameters.Add("end", end.ToString());

            return (await ExecuteAsync<Messages>(ConstruyePeticion(Config.SUBJECTS_URI + '/' + domainId + Config.BOARDS_SUB_URI + '/' + id + Config.MESSAGES_SUB_SUB_URI, parameters)));
        }

        public static async Task<Messages> GetInbox(int start, int end)
        {
            var parametros = new Dictionary<string, string>();

            parametros.Add("start", start.ToString());
            parametros.Add("end", end.ToString());

            return (await ExecuteAsync<Messages>(ConstruyePeticion(Config.MAILMESSAGES_URI, parametros)));
        }

        public static void GuardarCultura(CultureInfo cultura)
        {
            string SAVEFILENAME = "CULTURA";
            // Save the game state (in this case, the high score).
#if WINDOWS_PHONE
            IsolatedStorageFile savegameStorage = IsolatedStorageFile.GetUserStoreForApplication();
#else
            IsolatedStorageFile savegameStorage = IsolatedStorageFile.GetUserStoreForDomain();
#endif

            // open isolated storage, and write the savefile.
            IsolatedStorageFileStream fs = null;
            using (fs = savegameStorage.CreateFile(SAVEFILENAME))
            {
                if (fs != null)
                {
                    // just overwrite the existing info for this example.
                    byte[] bytes = System.Text.Encoding.UTF8.GetBytes(cultura.Name);
                    fs.Write(bytes, 0, bytes.Length);
                }
            }

        }

        public static CultureInfo ObtenerCultura()
        {
            string SAVEFILENAME = "CULTURA";
            // open isolated storage, and load data from the savefile if it exists.
#if WINDOWS_PHONE
            using (IsolatedStorageFile savegameStorage = IsolatedStorageFile.GetUserStoreForApplication())
#else
            using (IsolatedStorageFile savegameStorage = IsolatedStorageFile.GetUserStoreForDomain())
#endif
            {
                if (savegameStorage.FileExists(SAVEFILENAME))
                {
                    using (IsolatedStorageFileStream fs = savegameStorage.OpenFile(SAVEFILENAME, System.IO.FileMode.Open))
                    {
                        if (fs != null)
                        {
                            // Reload the saved high-score data.
                            byte[] saveBytes = new byte[5];
                            int count = fs.Read(saveBytes, 0, 5);
                            if (count > 0)
                            {
                                string cult = UTF8Encoding.UTF8.GetString(saveBytes, 0, 5); //BitConverter.ToString(saveBytes);
                                var cultura = new CultureInfo(cult);
                                return cultura;
                            }
                        }

                        return null;
                    }
                }
                else
                {
                    GuardarCultura(Thread.CurrentThread.CurrentCulture);
                    return Thread.CurrentThread.CurrentCulture;
                }

            }
        }

        public static void CargarIdioma()
        {
            Thread.CurrentThread.CurrentCulture = ObtenerCultura();
            Thread.CurrentThread.CurrentUICulture = Thread.CurrentThread.CurrentCulture;
        }
    }

}

