﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using CampusUOC.Clases;
using CampusUOC.Recursos;
using Microsoft.Phone.Controls;
using Newtonsoft.Json;
using RestSharp;
using WPControls;

namespace CampusUOC
{
    public partial class MenuPrincipal : PhoneApplicationPage
    {
        private User _user;
        private InboxMapping _inbox;
        private Classrooms _clases;
        private Eventos _eventos;

        public Eventos Events
        {
            get { return _eventos; }

            set
            {
                _eventos = value;

                List<DateTime> dates = new List<DateTime>();
                
                
                foreach (var evento in _eventos.events)
                {
                    DateTime fecha = new DateTime();
                    if (DateTime.TryParse(evento.start, out fecha))
                        dates.Add(fecha);
                }

                ((ColorConverter)Resources["ColorConverter"]).Dates = dates;
                
            }
        }

        public MenuPrincipal()
        {
            InitializeComponent();

            if (!ValidarLicencia())
                NavigationService.GoBack();

            Inicializar();
        }

        private async void Inicializar()
        {
            InicializarIdioma();
            var catalan = new CultureInfo("ca-ES");
            var ingles = new CultureInfo("en-GB");
            var castellano = new CultureInfo("es-ES");
            lpkIdioma.ItemsSource = new List<CultureInfo> { catalan, ingles, castellano };
            lpkIdioma.SelectedItem = Procedimientos.ObtenerCultura();
            //this.lpkIdioma.ItemsSource = new[] { "ca-ES", "en-GB", "es-ES" };

            lblTienesXMensajes.Text = AppResources.Loading + ": Inbox";
            try
            {
                this._inbox = await Procedimientos.GetInbox();
            }
            catch (Exception exc)
            {
                lblTienesXMensajes.Text = "Error cargando Inbox: " + (exc.InnerException == null ? exc.Message : exc.InnerException.Message);
            }
            
            lblTienesXMensajes.Text = AppResources.Loading + ": User";

            try
            {
                this._user = await Procedimientos.GetUser();
            }
            catch (Exception exc)
            {
                lblTienesXMensajes.Text = "Error cargando User: " + (exc.InnerException == null ? exc.Message : exc.InnerException.Message);
            }

            try
            {
                this._clases = await Procedimientos.GetClasses();
            }
            catch (Exception exc)
            {
                lblTienesXMensajes.Text = "Error cargando Classes: " + (exc.InnerException == null ? exc.Message : exc.InnerException.Message);
            }

            this.lblBienvenido.Text = AppResources.Welcome + " " + _user.name;
            this.lblTienesXMensajes.Text = AppResources.YouHave + " " + _inbox.unreadMessages + " " + AppResources.MessagesInYourInbox;
            try
            {
                if(!_user.photoUrl.Equals("http://cv.uoc.edu/UOC/mc-icons/fotos/desconegut.png"))
                    imagenAlumno.Source = new BitmapImage(new Uri(_user.photoUrl, UriKind.Absolute));
                else
                {
                    //throw new ArgumentNullException("Mejor poner un question mark a que no se vea nada");
                    imagenAlumno.Source = new BitmapImage(new Uri("Imagenes/questionmark.png", UriKind.Relative));
                }
            }
            catch (Exception nie)
            {
                try
                {
                    imagenAlumno.Source = new BitmapImage(new Uri("Imagenes/questionmark.png", UriKind.Relative));
                }
                catch (Exception)
                { }
            }

            try
            {
                Events = await Procedimientos.GetEventos();
            }
            catch (Exception exc)
            {
                lblTienesXMensajes.Text = "Error cargando Calendario: " + (exc.InnerException == null ? exc.Message : exc.InnerException.Message);
            }

            this.btnBuzon.Content = AppResources.Inbox + " (" + _inbox.unreadMessages + ")";
            

            lbAsignaturas.ItemsSource = this._clases.classrooms;
            
        }

        private void InicializarIdioma()
        {
            lblBienvenido.Text = AppResources.LoadingProgress;
            pvtAsignaturas.Header = AppResources.Subjects;
            pvtBuzon.Header = AppResources.Mail;
            pvtConfiguracion.Header = AppResources.Config;
            pvtPrincipal.Header = AppResources.Main;
            piCalendario.Header = AppResources.Calendar;
            btnComponerMail.Content = AppResources.NewMessage;
            
            if (_inbox != null) this.btnBuzon.Content = AppResources.Inbox + " (" + _inbox.unreadMessages + ")";
            //btnEnviados.Content = AppResources.SentMessages;
            if (_user != null) this.lblBienvenido.Text = AppResources.Welcome + " " + _user.name;
            if (_inbox != null) this.lblTienesXMensajes.Text = AppResources.YouHave + " " + _inbox.unreadMessages + " " + AppResources.MessagesInYourInbox;
        }

        /*
        private T ConstruyeClase<T>(string json) where T : new()
        {
            try
            {
                return (T)Newtonsoft.Json.JsonConvert.DeserializeObject(json, typeof(T));
            }
            catch (Exception exc)
            {
                MessageBox.Show("Problema recogiendo datos: " + exc.Message +
                                (exc.InnerException == null ? string.Empty : ". " + exc.InnerException.Message) +
                                ". Respuesta: " + json);
                return new T();
            }            
        }
        */



        private bool ValidarLicencia()
        {
            if (LicenciaActual.Licencia == null || LicenciaActual.Licencia.access_token == null)
            {
                Debug.WriteLine("access_token null");
                return false;
            }

            return true;
        }

        private void btnBuzon_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/Inbox.xaml", UriKind.Relative));
        }

        private void Button_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            Classroom c = (Classroom)((Button)sender).DataContext;
            NavigationService.Navigate(new Uri("/Asignatura.xaml?idSubject=" + c.id, UriKind.Relative));
        }

        private void PivotItem_GotFocus(object sender, RoutedEventArgs e)
        {
            
        }

        private void calendario_GotFocus(object sender, RoutedEventArgs e)
        {
           

        }

        private void calendario_DateClicked(object sender, WPControls.SelectionChangedEventArgs e)
        {
            if(Events == null || Events.events.Count == 0) return;

            DateTime fecha = new DateTime();
            var eventosSeleccionados =
                Events.events.Where(x => DateTime.TryParse(x.start, out fecha) && fecha.Date == e.SelectedDate);

            string salida = string.Empty;
            foreach (var eventoSeleccionado in eventosSeleccionados)
            {
                salida += eventoSeleccionado.summary + Environment.NewLine;
            }

            if(!salida.Equals(string.Empty))
                MessageBox.Show(salida);
        }

        private void lpkIdioma_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e)
        {
            if (lpkIdioma != null)
            {
                CultureInfo nuevoIdioma = (CultureInfo) lpkIdioma.SelectedItem;
                Procedimientos.GuardarCultura(nuevoIdioma);
                Procedimientos.CargarIdioma();
                InicializarIdioma();
            }
        }

        private void btnComponerMail_Click(object sender, RoutedEventArgs e)
        {
            NavigationService.Navigate(new Uri("/Compose.xaml", UriKind.Relative));
        }


    }
}